#!/usr/bin/python
# -!- coding: utf8
# Es geht um das Produkt „piups+“ der Fa. Ritter Elektronik, bei Reichelt 35€.
# Die Hardware, ATmega8-Firmware und RaspberryPi-Software ist closed-source.
# Leider bietet die Gesamtlösung weder eine Batterieanzeige (erwartungsgemäß
# in der GUI) noch ein automatisches Herunterfahren bei leerem Akku an.
# Stattdessen wird nach einer festen Zeit nach Stromausfall heruntergefahren,
# Vorgabe 5 s (also nur für einen kleinen Akku).
# Was funktioniert ist der Ein- und Ausschaltknopf; die Ausschaltfunktion
# läuft über I²C, die Einschaltfunktion durch Zuschaltung der 5 V.
# Dadurch keine Funktion der Taste in der Zeitspanne zwischen Herunterfahren
# und Stromabschaltung (müsste SCL nach Low ziehen), Firmware-Bug.
# Zwangsabschaltung wurde nicht getestet (= langes Festhalten der Taste).
# Die Taste ist ungünstig angeordnet und nicht per Pfostenstecker verlängerbar.
# Der Akku-Ladestrom ist auf 100 mA begrenzt.
# Durch Austausch eines 10-kΩ-Widerstandes gegen 2 kΩ kann dieser
# auf 500 mA erhöht werden, nicht durch Software.
# In den ATmega8 hätte man kinderleicht eine Echtzeituhr integrieren können.
# Einigermaßen dokumentiert ist nur die I²C-Schnittstelle:
# https://forum-raspberrypi.de/forum/thread/29737-pi-usv-monitoring-script/
import smbus
import sys
import time
import os

class PiUSV(object):
 def __init__(self):
  # Adresse
  self.address = 0x18
  self.piusv = smbus.SMBus(1)

 def get(self,what,len):
  self.piusv.write_byte(self.address,what)
  ret=[]
  for i in range(len):
   try:
    ret.append(self.piusv.read_byte(self.address))
   except (IOError, err):
    print ("Fehler beim Lesen von Device 0x{:02X}".format(self.address))
  return ret

  # Firmware-Version auslesen
 def get_version(self):
  return "".join([chr(item) for item in self.get(1,12)])

 # Statusbyte auslesen 
 def get_status(self):
  return self.get(0,1)[0]	# nur 1 Byte

 # Die Parameter der PIUSV+ byteweise auslesen
 def get_values(self):
  a = self.get(2,10)	# Daten kommen MSBfirst (ausgestorben aber typisch I²C)
  values = []
  for i in range(len(a)/2):
   values.append(256*a[i*2]+a[i*2+1])
  return values

 def printversion(self):
  # TODO: Gefährliche Zeichen escapen
  print ("Firmware-Version: „{:}“".format(self.get_version()))
  return 1

# Werte mit Namen versehen
 def printvalues(self):
  names = ["U_Batt", "I_Rasp", "U_Rasp", "U_USB", "U_ext"]
  values= self.get_values()
  units = ["V", "A", "V", "V", "V"]
  for i,v in enumerate(values):
   print("%7s:\33[1m%7.3f\33[0m %s" % (names[i], v/1000.0, units[i]))
  return len(values)	# Anzahl Zeilen für Cursor-Rücksprung

 # Statusbyte auswerten und ausgeben
 def printstatus(self):
  status = self.get_status()
  if status==0:
   return 0
  zeilen=1
  print ("Status")
  if status&0x01:
   print (" ----.---1 Spannung von USB-Buchse (5 V)")
   zeilen+=1
  if status&0x02:
   print (" ----.--1- Spannung von Uext (5..25 V)")
   zeilen+=1
  if status&0x04:
   print (" ----.-1-- Akku leer")
   zeilen+=1
  if status&0x08:
   print (" ----.1--- Akku wird geladen")
   zeilen+=1
  if status&0x10:
   print (" ---1.---- Akku voll")
   zeilen+=1
  if status&0x20:
   print (" --1-.---- Taster betätigt (= Shutdown)")
   zeilen+=1
  return zeilen

def main():
 # Script mit diversen Optionen abarbeiten
 piusv = PiUSV()
 try:
  option = sys.argv[1]
 except:
  piusv.printversion()
  while True:
   try:
    lines = piusv.printvalues() + piusv.printstatus()
    time.sleep(0.25)
    print("\33[{:d}A".format(lines+1))
   except:
    sys.exit(0)

 # Firmwareversion
 if option == "version": piusb.printversion()
 # Status und Bedeutung
 elif option == "status": piusv.printstatus()
 # Logdatei schreiben
 elif option == "log":
  log = time.strftime("%Y%m%d-%H%M%S")+" |"+ piusv.get_values() + piusv.get_status() + "\n"
  with open('/var/log/PIUSV.log', 'a') as fh:
   fh.write(log)

 # komplett
 elif option == "all":
  piusv.printvalues()
  piusv.printstatus()

 # Nur eine Zahl zur Weiterverarbeitung
 elif option == "U_Batt": print (piusv.get_values()[0])
 elif option == "I_Rasp": print (piusv.get_values()[1])
 elif option == "U_Rasp": print (piusv.get_values()[2])
 elif option == "U_USB":  print (piusv.get_values()[3])
 elif option == "U_ext":  print (piusv.get_values()[4])

 # Falsche Option, Hilfeseite ausgeben
 else:
  print ("Mögliche Optionen:")
  names=["version","status","all","log","U_Batt","I_Rasp","U_Rasp","U_USB","U_ext"]
  for name in names:
   print (" {}".format(name))

if __name__ == '__main__':
 main()
